!    DICE: DICE07 with variable time step
!    Copyright (C) 2010  Klaus Keller and Brian Tuttle 
!
!    This program is free software; you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation; either version 2 of the License, or
!    (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with this program; if not, write to the Free Software
!    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!
!    Klaus Keller, klaus.psu@gmail.com
!    Brian Tuttle, btuttle@psu.edu
!

!-----------------------------------------------------------------------------
MODULE dice_out

    USE global
    USE dice_in
    USE DICE
    USE dice_geo

    implicit none

    PRIVATE

! IO unit numbers:

! Time arrays:
    integer(i4b), parameter :: dice_yrs_ID          = 70

! Decision vector of optimization parameters:
    integer(i4b), parameter, public :: x_ID         = 90

! DICE economic variables:
    integer(i4b), parameter :: mu_ID                = 100
    integer(i4b), parameter :: IoY_ID               = 110
    integer(i4b), parameter :: utility_ID           = 120
    integer(i4b), parameter :: tax_ID               = 130
    integer(i4b), parameter :: production_ID        = 140
    integer(i4b), parameter :: emissions_ID         = 150
    integer(i4b), parameter :: abatement_ID         = 160
    integer(i4b), parameter :: per_cap_cons_ID      = 170
    integer(i4b), parameter :: GWP_ID               = 180
    integer(i4b), parameter :: consumption_ID       = 190
    integer(i4b), parameter :: population_ID        = 200
    integer(i4b), parameter :: damage_ID            = 210
    integer(i4b), parameter :: discount_ID          = 220
    integer(i4b), parameter :: TFP_ID               = 230
    integer(i4b), parameter :: capital_ID           = 240

! DICE geophysical model variables:
    integer(i4b), parameter :: atmos_CO2_ID         = 300
    integer(i4b), parameter :: temp_ID              = 310
    integer(i4b), parameter :: sinks_ID             = 320
    integer(i4b), parameter :: delta_CO2_ID         = 330
    integer(i4b), parameter :: rad_forcing_ID       = 340


    PUBLIC :: write_dice

CONTAINS
!------------------------------------------------------------------------------
SUBROUTINE write_dice(mu_in,IoY_in)

    implicit none

    real(DP), dimension(ns), intent(IN) :: mu_in, IoY_in
    character(len=7) :: IoY_fname
    integer(i4b) :: istat, i

    if (opt_type_in == "BAU_invest") then
        IoY_fname = 'IoY.bau'
    else 
        IoY_fname = 'IoY.dat'
    end if

    open(mu_ID, FILE='mu.dat', STATUS='REPLACE', ACTION='WRITE', &
            FORM='FORMATTED', IOSTAT=istat)
    do i = 1, ns
        write(mu_ID,"(F15.7)") mu_in(i)
    end do
    close(mu_ID)

    open(IoY_ID, FILE=IoY_fname, STATUS='REPLACE', ACTION='WRITE', &
            FORM='FORMATTED', IOSTAT=istat)
    do i = 1, ns
        write(IoY_ID,"(F15.7)") IoY_in(i)
    end do
    close(IoY_ID)

    open(production_ID,FILE='production.dat',STATUS='REPLACE',ACTION='WRITE', &
            FORM='FORMATTED', IOSTAT=istat)
    do i = 1, ns
        write(production_ID,"(F15.7)") production(i)
    end do
    close(production_ID)

    open(emissions_ID,FILE='emissions.dat',STATUS='REPLACE',ACTION='WRITE', &
            FORM='FORMATTED', IOSTAT=istat)
    do i = 1, ns
        write(emissions_ID,"(F15.7)") E(i)
    end do
    close(emissions_ID)
    
    open(GWP_ID,FILE='GWP.dat',STATUS='REPLACE',ACTION='WRITE', &
            FORM='FORMATTED', IOSTAT=istat)
    do i = 1, ns
        write(GWP_ID,"(F15.7)") Y(i)
    end do
    close(GWP_ID)
    
    open(consumption_ID,FILE='consumption.dat',STATUS='REPLACE',ACTION='WRITE',&
            FORM='FORMATTED', IOSTAT=istat)
    do i = 1, ns
        write(consumption_ID,"(F15.7)") C(i)
    end do
    close(consumption_ID)
    
    open(population_ID,FILE='population.dat',STATUS='REPLACE',ACTION='WRITE',&
            FORM='FORMATTED', IOSTAT=istat)
    do i = 1, ns
        write(population_ID,"(F15.7)") L(i)
    end do
    close(population_ID)

    open(damage_ID,FILE='damage.dat',STATUS='REPLACE',ACTION='WRITE',&
            FORM='FORMATTED', IOSTAT=istat)
    do i = 1, ns
        write(damage_ID,"(F15.7)") frac_damages(i)
    end do
    close(damage_ID)

    open(discount_ID,FILE='discount.dat',STATUS='REPLACE',ACTION='WRITE',&
            FORM='FORMATTED', IOSTAT=istat)
    do i = 1, ns
        write(discount_ID,"(F15.7)") RR(i)
    end do
    close(discount_ID)

    open(TFP_ID,FILE='TFP.dat',STATUS='REPLACE',ACTION='WRITE',&
            FORM='FORMATTED', IOSTAT=istat)
    do i = 1, ns
        write(TFP_ID,"(F15.7)") AL(i)
    end do
    close(TFP_ID)

    open(capital_ID,FILE='capital.dat',STATUS='REPLACE',ACTION='WRITE',&
            FORM='FORMATTED', IOSTAT=istat)
    do i = 1, ns
        write(capital_ID,"(F15.7)") K(i)
    end do
    close(capital_ID)

    open(atmos_CO2_ID,FILE='atmos_CO2.dat',STATUS='REPLACE',ACTION='WRITE',&
            FORM='FORMATTED', IOSTAT=istat)
    do i = 1, ns
        write(atmos_CO2_ID,"(F15.7)") M_AT(i)
    end do
    close(atmos_CO2_ID)

    open(temp_ID,FILE='temp.dat',STATUS='REPLACE',ACTION='WRITE',&
            FORM='FORMATTED', IOSTAT=istat)
    do i = 1, ns
        write(temp_ID,"(F15.7)") T_ATM(i)
    end do
    close(temp_ID)

    open(rad_forcing_ID,FILE='rad_forcing.dat',STATUS='REPLACE',ACTION='WRITE',&
            FORM='FORMATTED', IOSTAT=istat)
    do i = 1, ns
        write(rad_forcing_ID,"(F15.7)") FORC(i)
    end do
    close(rad_forcing_ID)

    open(dice_yrs_ID, FILE='dice_yrs.dat', &
         STATUS='REPLACE', ACTION='WRITE', ACCESS='SEQUENTIAL', IOSTAT=istat)
    do i = 1, ns
        write(dice_yrs_ID,"(F15.7)") time_yrs(i)
    end do
    close(dice_yrs_ID, IOSTAT=istat)

    RETURN

END SUBROUTINE write_dice
!------------------------------------------------------------------------------
END MODULE dice_out
