!    RNDSEED
!    Copyright (C) 2010  Klaus Keller and Brian Tuttle 
!
!    This program is free software; you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation; either version 2 of the License, or
!    (at your option) any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with this program; if not, write to the Free Software
!    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!
!    Klaus Keller, klaus.psu@gmail.com
!    Brian Tuttle, btuttle@psu.edu
!
!-----------------------------------------------------------------------------
MODULE rndseed

    USE global

    implicit none

    private
    integer(i4b), parameter, public :: nRT = 10
    integer(i4b), parameter, public :: max_rnd_seed = 100000000

    public :: randate, rnd_dist

    interface rnd_dist
      module procedure rnd_d1, rnd_d2, rnd_d3
    end interface

CONTAINS
!---------------------------------------------------------------------------
FUNCTION randate() RESULT(rnd)
!===========================================================================
!   Generate a pseudo-random number out of the date_and_time() subroutine.
!   Brian Tuttle 25Feb05 <btuttle@psu.edu>
!===========================================================================

    implicit none

    integer :: rndint, maxrnd, rnd
    integer :: i, dig1, dig2, dig3, num1, num2, num3
    double precision :: nrnd
    character(len=10) :: timestr

    num1 = 0
    num2 = 0
    num3 = 0
! Get the time ("hhmmss.sss")
    call date_and_time(time=timestr)
! Convert the string into groups of three digits, avoiding the '.'
    do i=1,3
       dig1 =  ichar(timestr(i:i)) - ichar('0')
       dig2 =  ichar(timestr(i+3:i+3)) - ichar('0')
       dig3 =  ichar(timestr(i+7:i+7)) - ichar('0')
       num1 = 10*num1 + dig1
       num2 = 10*num2 + dig2
       num3 = 10*num3 + dig3
    end do
! Multiply them and add the last digit for good measure.
    rndint = num1*num2*num3 + dig3
! Normalize the result.
    maxrnd = 235 * 959 * 999 + 9
    nrnd = real(rndint)/real(maxrnd)
! Make the result a nine digit number max.
    rnd = int(max_rnd_seed * nrnd)

    RETURN

END FUNCTION randate
!---------------------------------------------------------------------------

!---------------------------------------------------------------------------
SUBROUTINE rnd_d1(randout)
!  =======================================================================
! |  Generates a one-dimensional array of random numbers                  |
!  =======================================================================

    USE mt19937

    implicit none

    double precision, dimension(:), intent(OUT) :: randout
    integer :: i

    do i = 1,size(randout,1)
        randout(i) = genrand_real1()
    end do

    RETURN

END SUBROUTINE rnd_d1
!---------------------------------------------------------------------------

!---------------------------------------------------------------------------
SUBROUTINE rnd_d2(randout)
!  =======================================================================
! |  Generates a two-dimensional array of random numbers                  |
!  =======================================================================

    USE mt19937

    implicit none

    double precision, dimension(:,:), intent(OUT) :: randout
    integer :: i, j

    do i = 1,size(randout,1)
      do j = 1,size(randout,2)
        randout(i,j) = genrand_real1()
      end do
    end do

    RETURN

END SUBROUTINE rnd_d2
!---------------------------------------------------------------------------

!---------------------------------------------------------------------------
SUBROUTINE rnd_d3(randout)
!  =======================================================================
! |  Generates a three-dimensional array of random numbers                |
!  =======================================================================

    USE mt19937

    implicit none

    double precision, dimension(:,:,:), intent(OUT) :: randout
    integer :: i, j, k

    do i = 1,size(randout,1)
      do j = 1,size(randout,2)
        do k = 1,size(randout,3)
          randout(i,j,k) = genrand_real1()
        end do
      end do
    end do

    RETURN

END SUBROUTINE rnd_d3
!---------------------------------------------------------------------------
END MODULE rndseed
